/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan;

import java.awt.Dimension;
import java.awt.Toolkit;

import java.io.File;

import nan.view.View;

/**
 * The main starting class for NaN.
 *
 * @author John David Ratliff
 * @version 1.24, 1/10/04
 */
public class NaN {
    /**
     * The application name String.
     */
    public static final String APP_NAME = "NaN - Never Again N@tscape";

    /**
     * The application version String.
     */
    public static final String APP_VERSION = "v2.8.210";

    /**
     * The application copyright String.
     */
    public static final String APP_COPYRIGHT = "Copyright (C) 2003 John David Ratliff";

    /**
     * The count of files in the source code.
     */
    public static final String FILE_COUNT = "72";

    /**
     * The count of lines in the source code.
     */
    public static final String LINE_COUNT = "14,659";

    /**
     * The documentation file.
     */
    public static final String HELP_FILE = "documentation" + File.separator +
                                           "internal.html";

    /**
     * The base directory into which all NaN files will be stored.
     */
    public static final String BASE_DIRECTORY = System.getProperty("user.home") +
                                                File.separator + ".nan";

    /**
     * The directory into which setting files will be stored.
     */
    public static final String SETTINGS_DIRECTORY = "settings";

    /**
     * The directory into which logs will be stored.
     */
    public static final String LOG_DIRECTORY = "logs";

    /**
     * The directory into which message caches will be stored.
     */
    public static final String CACHE_DIRECTORY = "cache";

    /**
     * The directory into which license agreements will be stored.
     */
    public static final String LICENSE_DIRECTORY = "license";

    /**
     * The license file for the main free license terms.
     */
    public static final String FREE_LICENSE = "Free License";

    /**
     * The license file for the MindTerm Public Source license.
     */
    public static final String MINDTERM_SOURCE_LICENSE = "MindTerm Public Source License";

    /**
     * The license file for the MindTerm End-User license.
     */
    public static final String MINDTERM_END_USER_LICENSE = "MindTerm End-User License";

    /**
     * The file into which the free license agreement will be stored.
     */
    public static final String FREE_FILE = "free.xml";

    /**
     * The file into which the MindTerm Public Source license agreement will
     * be stored.
     */
    public static final String MINDTERM_SOURCE_FILE = "mindterm-source.xml";

    /**
     * The file into which the MindTerm End-User license agreement will be
     * stored.
     */
    public static final String MINDTERM_END_USER_FILE = "mindterm-end-user.xml";

    /**
     * The file into which User Settings are stored.
     */
    public static final String USER_SETTINGS_FILE = "user.xml";

    /**
     * The file into which Server Settings are stored.
     */
    public static final String SERVER_SETTINGS_FILE = "server.xml";

    /**
     * The file into which Available Newsgroups are stored.
     */
    public static final String AVAILABLE_NEWSGROUPS_FILE = "available.xml";

    /**
     * The file into which Subscribed Newsgroups are stored.
     */
    public static final String SUBSCRIBED_NEWSGROUPS_FILE = "subscribed.xml";

    /**
     * The name of the archive into which the settings will be stored for
     * remote saving.
     */
    public static final File ARCHIVE_FILE = new File(BASE_DIRECTORY,
                                                     "nan-settings.zip");

    /**
     * The default window width.
     */
    public static final int DEFAULT_WIDTH;

    /**
     * The default window height.
     */
    public static final int DEFAULT_HEIGHT;

    static {
        // setup the default width/height to 10% less than the screensize
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        Dimension size = toolkit.getScreenSize();

        DEFAULT_WIDTH = (int)size.getWidth() -
                        (int)(size.getWidth() * 0.10);
        DEFAULT_HEIGHT = (int)size.getHeight() -
                         (int)(size.getHeight() * 0.10);
    }

    /**
     * Creates a new NaN object.
     */
    private NaN() {
        // create the NaN directories
        File directory = new File(BASE_DIRECTORY);

        if (!directory.exists()) {
            directory.mkdir();
        }

        directory = new File(BASE_DIRECTORY + File.separator +
                             SETTINGS_DIRECTORY);

        if (!directory.exists()) {
            directory.mkdir();
        }

        directory = new File(BASE_DIRECTORY + File.separator +
                             LOG_DIRECTORY);

        if (!directory.exists()) {
            directory.mkdir();
        }

        directory = new File(BASE_DIRECTORY + File.separator +
                             CACHE_DIRECTORY);

        if (!directory.exists()) {
            directory.mkdir();
        }

        directory = new File(BASE_DIRECTORY + File.separator +
                             LICENSE_DIRECTORY);

        if (!directory.exists()) {
            directory.mkdir();
        }

        // set mac look and feel for menus on OS-X
        System.setProperty("com.apple.laf.useScreenMenuBar", "true");

        // create and display the view
        View view = new View(DEFAULT_WIDTH, DEFAULT_HEIGHT);
        view.setVisible(true);
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return APP_NAME + ' ' + APP_VERSION + '\n' + APP_COPYRIGHT;
    }

    /**
     * Called when the program is started.
     *
     * @param args not currently used.
     */
    public static void main(String[] args) {
        new NaN();
    }
}
