/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.controller;

import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

/**
 * A popup menu for the attachments on a message.
 *
 * @author John David Ratliff
 * @version 1.3, 1/6/04
 */
public class AttachmentPopupMenu extends JPopupMenu {
    private ActionListener responder;

    /**
     * Creates a new AttachmentPopupMenu object.
     *
     * @param responder The ActionListener that responds to the items on the
     *        menu.
     */
    public AttachmentPopupMenu(ActionListener responder) {
        this.responder = responder;
    }

    /**
     * Adds an attachment to the menu.
     *
     * @param name The name on the menu, which is also set as the action
     *        command.
     */
    public void addAttachment(String name) {
        JMenuItem item = new JMenuItem(name);
        item.setActionCommand(name);
        item.addActionListener(responder);

        add(item);
    }

    /**
     * Returns a String representation of this menu.
     *
     * @return A String representation.
     */
    public String toString() {
        return "AttachmentPopupMenu[responder=" + responder + "]";
    }
}
