/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.controller;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import nan.tools.EventLogger;

import nan.view.View;

/**
 * Popup Menu used on the subscribed Newsgroup List to refresh the cache.
 *
 * @author John David Ratliff
 * @version 1.4, 1/3/04
 */
public class NewsgroupPopupMenu extends JPopupMenu implements ActionListener {
    private View view;
    private static final String REFRESH_ACTION = "_refresh";
    private static final String MESSAGES_ACTION = "_messages";
    private static final String DOWNLOAD_ACTION = "_download";

    /**
     * Creates a new NewsgroupPopupMenu object.
     *
     * @param view The View that handles the work.
     */
    public NewsgroupPopupMenu(View view) {
        super();

        this.view = view;

        JMenuItem refresh = new JMenuItem("Refresh Current Newsgroup");
        refresh.setActionCommand(REFRESH_ACTION);
        refresh.addActionListener(this);

        add(refresh);

        JMenuItem messages = new JMenuItem("Set Messages to Cache");
        messages.setActionCommand(MESSAGES_ACTION);
        messages.addActionListener(this);

        add(messages);

        JMenuItem download = new JMenuItem("Download Full Messages");
        download.setActionCommand(DOWNLOAD_ACTION);
        download.addActionListener(this);

        add(download);
    }

    /**
     * Called when the refresh action is selected.
     */
    private void refreshAction() {
        this.view.refreshCurrentNewsgroup();
    }

    /**
     * Called when the messages action is selected.
     */
    private void messagesAction() {
        this.view.changeNewsgroupMessages();
    }

    /**
     * Called when the download action is selected.
     */
    private void downloadAction() {
        this.view.cacheCurrentNewsgroup();
    }

    /**
     * The registered callback for events on this menu.
     *
     * @param event The ActionEvent that triggered this callback.
     */
    public void actionPerformed(ActionEvent event) {
        String action = event.getActionCommand();

        if (action.equals(REFRESH_ACTION)) {
            refreshAction();
        } else if (action.equals(MESSAGES_ACTION)) {
            messagesAction();
        } else if (action.equals(DOWNLOAD_ACTION)) {
            downloadAction();
        } else {
            EventLogger.getInstance().logError("Unknown action command " +
                                               action + " seen in " +
                                               "NewsgroupPopupMenu.actionPerformed()...");
        }
    }

    /**
     * Returns a String representation of this Menu.
     *
     * @return A String representation.
     */
    public String toString() {
        return "NewsgroupPopupMenu[view=" + this.view + "]";
    }
}
