/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.tools;

import java.io.File;

/**
 * Background task that deletes a directory and all files under it.
 *
 * @author John David Ratliff
 * @version 1.6, 1/6/04
 */
public class BackgroundDirectoryDeleter extends AbstractBackgroundTask {
    private File directory;

    /**
     * Creates a new BackgroundDirectoryDeleter object.
     *
     * @param directory The directory to delete.
     */
    public BackgroundDirectoryDeleter(File directory) {
        this.directory = directory;
    }

    /**
     * Deletes a directory and any files and subdirectories under it
     * recursively.
     *
     * @param directory The directory to delete.
     */
    private void deleteDirectory(File directory) {
        File[] files = directory.listFiles();

        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                File file = files[i];

                if (file.isDirectory()) {
                    deleteDirectory(directory);
                }

                boolean success = file.delete();

                if (!success) {
                    EventLogger.getInstance().logError("Unable to delete file " +
                                                       file +
                                                       " in BackgroundDirectoryDeleter().deleteDirectory()...");
                }
            }
        }
    }

    /**
     * Starts this delete task.
     */
    public void run() {
        try {
            deleteDirectory(directory);
            directory.delete();
        } catch (Exception e) {
            // don't let an Exception crash us!
            EventLogger.getInstance().logError(e.toString() +
                                               " during BackgroundDirectoryDeleter.run()...");
        }

        finished = true;
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "BackgroundDirectoryDeleter[directory=" + directory + "]";
    }
}
