/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.view;

import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.File;

import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import nan.NaN;

import nan.controller.ComposeMenu;

import nan.model.Attachment;
import nan.model.Message;
import nan.model.MessageHeader;

import nan.tools.EventLogger;

/**
 * A frame for composing messages.
 *
 * @author John David Ratliff
 * @version 1.17, 1/6/03
 */
public class ComposeFrame extends JFrame implements ActionListener {
    private JPanel buttonPanel;
    private JButton cancelButton;
    private JEditorPane editor;
    private JScrollPane editorScroller;
    private JLabel from;
    private JLabel fromLabel;
    private JTextField newsgroups;
    private JLabel newsgroupsLabel;
    private JButton sendButton;
    private JSeparator separator;
    private JSeparator separator2;
    private JTextField subject;
    private JLabel subjectLabel;
    private View parent;
    private JLabel attachmentsLabel;
    private JLabel attachments;
    private String references;
    private ComposeMenu menu = new ComposeMenu(this);
    private Message message = new Message();
    private static final String SEND_ACTION = "_send";
    private static final String CANCEL_ACTION = "_cancel";

    /**
     * Creates a new ComposeFrame object.
     *
     * @param parent The parent of this frame.
     */
    public ComposeFrame(View parent) {
        this.parent = parent;

        initComponents();
        setJMenuBar(menu);
    }

    /**
     * Initalizes the frame components.
     */
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        buttonPanel = new JPanel();
        sendButton = new JButton();
        cancelButton = new JButton();
        separator = new JSeparator();
        fromLabel = new JLabel();
        from = new JLabel();
        newsgroupsLabel = new JLabel();
        newsgroups = new JTextField();
        subjectLabel = new JLabel();
        subject = new JTextField();
        attachmentsLabel = new JLabel();
        attachments = new JLabel();
        separator2 = new JSeparator();
        editorScroller = new JScrollPane();
        editor = new JEditorPane();

        getContentPane().setLayout(new GridBagLayout());

        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Message Composer");
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent evt) {
                    cancelButtonAction();
                }
            });

        sendButton.setMnemonic('s');
        sendButton.setText("Send");
        sendButton.setActionCommand(SEND_ACTION);
        sendButton.addActionListener(this);
        buttonPanel.add(sendButton);

        cancelButton.setMnemonic('c');
        cancelButton.setText("Cancel");
        cancelButton.setActionCommand(CANCEL_ACTION);
        cancelButton.addActionListener(this);
        buttonPanel.add(cancelButton);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new Insets(5, 5, 5, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        getContentPane().add(buttonPanel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 15, 5, 15);
        gridBagConstraints.weightx = 0.8;
        getContentPane().add(separator, gridBagConstraints);

        fromLabel.setText("From: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(fromLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 5);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(from, gridBagConstraints);

        newsgroupsLabel.setDisplayedMnemonic('n');
        newsgroupsLabel.setLabelFor(newsgroups);
        newsgroupsLabel.setText("Newsgroups: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(newsgroupsLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 5);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(newsgroups, gridBagConstraints);

        subjectLabel.setDisplayedMnemonic('u');
        subjectLabel.setLabelFor(subject);
        subjectLabel.setText("Subject: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(subjectLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 5);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(subject, gridBagConstraints);

        attachmentsLabel.setText("Attachments: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.05;
        getContentPane().add(attachmentsLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 0, 0, 5);
        gridBagConstraints.weightx = 0.95;
        getContentPane().add(attachments, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(10, 15, 5, 15);
        gridBagConstraints.weightx = 0.8;
        getContentPane().add(separator2, gridBagConstraints);

        editor.setFont(new Font("Monospaced", 0, 14));
        editorScroller.setViewportView(editor);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(editorScroller, gridBagConstraints);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width - 800) / 2,
                  (screenSize.height - 600) / 2, 800, 600);
    }

    /**
     * Called when the send button is pressed.
     */
    public void sendButtonAction() {
        MessageHeader header = new MessageHeader();
        header.addHeader("X-Newsreader", NaN.APP_NAME + " " +
                         NaN.APP_VERSION);
        header.addHeader("From", this.from.getText());
        header.addHeader("Subject", this.subject.getText());
        header.addHeader("Newsgroups", this.newsgroups.getText());

        if (references != null) {
            header.addHeader("References", references);
        }

        message.setHeader(header);
        message.setBody(editor.getText());

        if (!this.parent.getNNTPClient().post(message)) {
            EventLogger.getInstance().logError("Unable to post message in " +
                                               "View.postMessage()...");

            JOptionPane.showMessageDialog(this,
                                          "Unable to post your message.",
                                          "Error posting message.",
                                          JOptionPane.ERROR_MESSAGE);
        }

        // close this dialog and dispose of the resources
        this.setVisible(false);
        this.dispose();
    }

    /**
     * Called when the cancel button is pressed.
     */
    public void cancelButtonAction() {
        this.setVisible(false);
        this.dispose();
    }

    /**
     * Called when an attachment is requested.
     */
    public void addAttachmentAction() {
        String title = "Add Attachment";
        FileDialog dialog = new FileDialog(this, title, FileDialog.LOAD);
        dialog.setVisible(true);

        if (dialog.getFile() != null) {
            File file = new File(dialog.getDirectory(), dialog.getFile());
            Attachment attachment = Attachment.encode(file);
            message.addAttachment(attachment);

            menu.addAttachment(dialog.getFile());

            // rebuild the attachments label
            buildAttachmentsLabel();
        }
    }

    /**
     * Called to remove an attachment.
     *
     * @param name The name of the attachment to remove.
     */
    public void removeAttachmentAction(String name) {
        if (message != null) {
            for (int i = 0; i < message.getAttachmentCount(); i++) {
                Attachment attachment = message.getAttachment(i);

                if (name.equals(attachment.getName())) {
                    message.removeAttachment(i);
                    i = message.getAttachmentCount();

                    // rebuild the label
                    buildAttachmentsLabel();
                }
            }
        }
    }

    /**
     * Builds the attachments label.
     */
    private void buildAttachmentsLabel() {
        String text = "";

        if (message != null) {
            for (int i = 0; i < message.getAttachmentCount(); i++) {
                String name = message.getAttachment(i).getName();

                if (text.length() == 0) {
                    text = name;
                } else {
                    text += (", " + name);
                }
            }
        }

        attachments.setText(text);
    }

    /**
     * Displays this frame.
     *
     * @param from Who this message is from.
     * @param newsgroup The newsgroup this message is addressed to.
     * @param subject The subject of this Message.
     * @param signature The user's signature.
     * @param body The body of the Message.
     * @param references The message references for reply-to messages.
     */
    public void display(String from, String newsgroup, String subject,
                        String signature, String body, String references) {
        this.references = references;
        this.from.setText(from);
        this.newsgroups.setText(newsgroup);
        this.subject.setText(subject);

        if (signature.length() > 0) {
            body += ("\n\n" + signature);
            this.editor.setText(body);
            this.editor.setCaretPosition(0);
        }

        this.setVisible(true);
    }

    /**
     * Displays this frame.
     *
     * @param from Who this message is from.
     * @param newsgroup The newsgroup this message is addressed to.
     * @param signature The user's signature.
     */
    public void display(String from, String newsgroup, String signature) {
        display(from, newsgroup, "", signature, "", null);
    }

    /**
     * The registered callback for actions on this frame.
     *
     * @param event The event that triggered this callback.
     */
    public void actionPerformed(ActionEvent event) {
        String action = event.getActionCommand();

        if (action.equals(SEND_ACTION)) {
            sendButtonAction();
        } else if (action.equals(CANCEL_ACTION)) {
            cancelButtonAction();
        } else {
            EventLogger.getInstance().logError("Invalid action command " +
                                               action +
                                               " in ComposeFrame.actionPerformed()...");
        }
    }

    /**
     * Returns a String representation of this ComposeFrame.
     *
     * @return A String representation.
     */
    public String toString() {
        return "ComposeFrame[from=" + this.from + ", newsgroups=" +
               this.newsgroups + ", editor=" + this.editor + "]";
    }
}
