/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.view;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.FileReader;
import java.io.IOException;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;

import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import nan.NaN;

import nan.tools.EventLogger;

/**
 * A frame to display HTML help pages.
 *
 * @author John David Ratliff
 * @version 1.3, 1/7/04
 */
public class HelpFrame extends JFrame implements HyperlinkListener {
    private JEditorPane helpPane;
    private JScrollPane helpScroller;

    /**
     * Creates a new HelpFrame object.
     */
    public HelpFrame() {
        initComponents();
    }

    /**
     * Initializes the frame components.
     */
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        helpScroller = new JScrollPane();
        helpPane = new JEditorPane();

        getContentPane().setLayout(new GridBagLayout());

        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("NaN Documentation");
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent event) {
                    closeAction();
                }
            });

        helpPane.setContentType("text/html");

        try {
            FileReader reader = new FileReader(NaN.HELP_FILE);
            helpPane.read(reader, null);
            reader.close();
        } catch (IOException e) {
            EventLogger.getInstance().logError("IOException during HelpFrame.initComponents()...");
        }

        helpPane.setEditable(false);
        helpPane.addHyperlinkListener(this);
        helpScroller.setViewportView(helpPane);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(helpScroller, gridBagConstraints);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width - 640) / 2,
                  (screenSize.height - 480) / 2, 640, 480);
    }

    /**
     * Called when this frame is closed.
     */
    private void closeAction() {
        this.setVisible(false);
    }

    /**
     * Called when a hyperlink is selected.
     *
     * @param event The Hyperlink Event.
     */
    public void hyperlinkUpdate(HyperlinkEvent event) {
        if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            helpPane.scrollToReference(event.getDescription());
        }
    }

    /**
     * Returns a String representation of this object.
     *
     * @return A String representation.
     */
    public String toString() {
        return "HelpFrame[scroll pane=" + helpScroller + ", editor pane=" +
               helpPane + "]";
    }
}
