/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.view;

import java.awt.Component;
import java.awt.Font;

import javax.swing.JTable;

import javax.swing.table.DefaultTableCellRenderer;

import nan.model.MessageHeader;
import nan.model.Selected;

/**
 * A renderer for subject cells.
 *
 * @author John David Ratliff
 * @version 1.4, 1/3/04
 */
public class SubjectTableCellRenderer extends DefaultTableCellRenderer {
    private Font bold;
    private Font normal;

    /**
     * Creates a new SubjectTableCellRenderer object.
     */
    public SubjectTableCellRenderer() {
        this.normal = new Font(getFont().getName(), Font.PLAIN,
                               getFont().getSize());
        this.bold = new Font(normal.getName(), Font.BOLD, normal.getSize());
    }

    /**
     * Sets the value of this cell.
     *
     * @param object The object.
     */
    protected void setValue(Object object) {
        MessageHeader header = (MessageHeader)object;

        if (header.isRead()) {
            setFont(normal);
        } else {
            setFont(bold);
        }

        setText(header.getValue("subject"));
    }

    /**
     * Gets the component used for rendering a cell.
     *
     * @param table The JTable.
     * @param object The object being rendered.
     * @param isSelected If the cell is selected.
     * @param hasFocus If the cell has focus.
     * @param row The row in the table of this cell.
     * @param column The column in the table of this cell.
     *
     * @return The Component that will be used to paint this cell.
     */
    public Component getTableCellRendererComponent(JTable table,
                                                   Object object,
                                                   boolean isSelected,
                                                   boolean hasFocus,
                                                   int row, int column) {
        Selected selected = (Selected)object;

        if (selected.isRead()) {
            setFont(normal);
        } else {
            setFont(bold);
        }

        return super.getTableCellRendererComponent(table, object,
                                                   isSelected, hasFocus,
                                                   row, column);
    }

    /**
     * Returns a String representation of this renderer.
     *
     * @return A String representation.
     */
    public String toString() {
        return "SubjectTableCellRenderer[normal font=" + normal +
               ", bold font=" + bold + "]";
    }
}
