/*
 * Copyright (C) 2003-2004 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package nan.view;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Toolkit;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.WindowConstants;

/**
 * A dialog used for wait messages that cannot be closed by the user.
 *
 * @author John David Ratliff
 * @version 1.6, 12/28/03
 */
public class WaitDialog extends JDialog {
    private JLabel label;
    private String title;
    private String message;

    /**
     * Creates a new WaitDialog object.
     *
     * @param frame The parent frame of this dialog.
     */
    public WaitDialog(Frame frame) {
        super(frame, true);

        initComponents();
    }

    /**
     * Initializes the dialog components.
     */
    private void initComponents() {
        label = new JLabel();

        setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
        getContentPane().setLayout(new FlowLayout());
        getContentPane().add(label);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width - 300) / 2,
                  (screenSize.height - 75) / 2, 300, 75);
    }

    /**
     * Displays this dialog with a title and a message.
     *
     * @param title The title of the dialog.
     * @param message The message of the dialog.
     */
    public void display(String title, String message) {
        setTitle(title);
        label.setText(message);
        setVisible(true);
    }

    /**
     * Sets the message text of this dialog.
     *
     * @param text The new text.
     */
    public void setText(String text) {
        label.setText(text);
    }

    /**
     * Returns a String representation of this dialog.
     *
     * @return A String representation.
     */
    public String toString() {
        return "WaitDialog[label=" + label + ", title=" + title +
               ", message=" + message + "]";
    }
}
